@@ -178,13 +178,13 @@ fsk_bit_analyze( fsk_plan *fskp, float *samples, unsigned int bit_nsamples,
 static float
 fsk_frame_analyze( fsk_plan *fskp, float *samples, float samples_per_bit,
 	int n_bits, const char *expect_bits_string,
-	unsigned int *bits_outp, float *ampl_outp )
+	unsigned long long *bits_outp, float *ampl_outp )
 {
     unsigned int bit_nsamples = (float)(samples_per_bit + 0.5);
 
-    unsigned int	bit_values[32];
-    float		bit_sig_mags[32];
-    float		bit_noise_mags[32];
+    unsigned int	bit_values[64];
+    float		bit_sig_mags[64];
+    float		bit_noise_mags[64];
     unsigned int	bit_begin_sample;
     int			bitnum;
 
@@ -269,6 +269,8 @@ fsk_frame_analyze( fsk_plan *fskp, float *samples, float samples_per_bit,
 #if CONFIDENCE_ALGO == 5 || CONFIDENCE_ALGO == 6
 
     float total_bit_sig = 0.0, total_bit_noise = 0.0;
+    float avg_mark_sig = 0.0, avg_space_sig = 0.0;
+    unsigned int n_mark = 0, n_space = 0;
     for ( bitnum=0; bitnum<n_bits; bitnum++ ) {
 	// Deal with floating point data type quantization noise...
 	// If total_bit_noise <= FLT_EPSILON, then assume it to be 0.0,
@@ -276,6 +278,14 @@ fsk_frame_analyze( fsk_plan *fskp, float *samples, float samples_per_bit,
 	total_bit_sig   += bit_sig_mags[bitnum];
 	if ( bit_noise_mags[bitnum] > FLT_EPSILON )
 	    total_bit_noise += bit_noise_mags[bitnum];
+
+	if ( bit_values[bitnum] == 1 ) {
+	    avg_mark_sig += bit_sig_mags[bitnum];
+	    n_mark++;
+	} else {
+	    avg_space_sig += bit_sig_mags[bitnum];
+	    n_space++;
+	}
     }
 
     // Compute the "frame SNR"
@@ -284,12 +294,18 @@ fsk_frame_analyze( fsk_plan *fskp, float *samples, float samples_per_bit,
     // Compute avg bit sig and noise magnitudes
     float avg_bit_sig   = total_bit_sig / n_bits;
 
+    // Compute separate avg bit sig for mark and space
+    if ( n_mark )
+	avg_mark_sig /= n_mark;
+    if ( n_space )
+	avg_space_sig /= n_space;
+
 #if CONFIDENCE_ALGO == 6
     // Compute average "divergence": bit_mag_divergence / other_bits_mag
     float divergence = 0.0;
     for ( bitnum=0; bitnum<n_bits; bitnum++ ) {
-	float avg_bit_sig_other = (total_bit_sig - fabs(bit_sig_mags[bitnum]))
-					/ (n_bits - 1);
+	float avg_bit_sig_other;
+	avg_bit_sig_other = bit_values[bitnum] ? avg_mark_sig : avg_space_sig;
 	divergence += fabs(bit_sig_mags[bitnum] - avg_bit_sig_other)
 					/ avg_bit_sig_other;
     }
@@ -422,7 +438,7 @@ fsk_frame_analyze( fsk_plan *fskp, float *samples, float samples_per_bit,
     // into the bits_outp word.
     *bits_outp = 0;
     for ( bitnum=0; bitnum<n_bits; bitnum++ )
-	*bits_outp |= bit_values[bitnum] << bitnum;
+	*bits_outp |= (unsigned long long) bit_values[bitnum] << bitnum;
 
     debug_log("    frame algo=%u confidence=%f ampl=%f\n",
 	    CONFIDENCE_ALGO, confidence, *ampl_outp);
@@ -437,7 +453,7 @@ fsk_find_frame( fsk_plan *fskp, float *samples, unsigned int frame_nsamples,
 	unsigned int try_step_nsamples,
 	float try_confidence_search_limit,
 	const char *expect_bits_string,
-	unsigned int *bits_outp,
+	unsigned long long *bits_outp,
 	float *ampl_outp,
 	unsigned int *frame_start_outp
 	)
@@ -450,7 +466,7 @@ fsk_find_frame( fsk_plan *fskp, float *samples, unsigned int frame_nsamples,
 
     unsigned int best_t = 0;
     float best_c = 0.0, best_a = 0.0;
-    unsigned int best_bits = 0;
+    unsigned long long best_bits = 0;
     
     // Scan the frame positions starting with the one try_first_sample,
     // alternating between a step above that, a step below that, above, below,
@@ -466,7 +482,7 @@ fsk_find_frame( fsk_plan *fskp, float *samples, unsigned int frame_nsamples,
 	    continue;
 
 	float c, ampl_out;
-	unsigned int bits_out = 0;
+	unsigned long long bits_out = 0;
 	debug_log("try fsk_frame_analyze at t=%d\n", t);
 	c = fsk_frame_analyze(fskp, samples+t, samples_per_bit,
 			expect_n_bits, expect_bits_string,
